@@ -1,10 +1,10 @@
 from rest_framework import status
 from rest_framework.decorators import api_view
-from rest_framework.request import HttpRequest
+from rest_framework.request import Request
 from rest_framework.response import Response
 from rest_framework.views import APIView
 from rest_framework.viewsets import ModelViewSet
-from . import models, serializers, get_task_model, get_schedule_log_model
+from . import serializers, get_task_model, get_schedule_log_model
 from .models import TaskSchedule
 from .choices import TaskScheduleStatus
 from django_common_objects.rest_view import UserListAPIView, UserRetrieveAPIView
@@ -16,12 +16,27 @@ from threading import Lock
 
 
 schedule_queue_lock = Lock()
-schedule_queue = Queue()
-
 TaskModel = get_task_model()
 ScheduleLogModel = get_schedule_log_model()
 
 
+status_params_mapping = {
+    'opening': TaskScheduleStatus.OPENING.value,
+    'closed': TaskScheduleStatus.CLOSED.value,
+    'test': TaskScheduleStatus.TEST.value,
+    'done': TaskScheduleStatus.DONE.value,
+    'error': TaskScheduleStatus.ERROR.value,
+}
+
+
+class QueueMapping(dict):
+
+    def __init__(self, keys):
+        super().__init__()
+        for key in keys:
+            self[key] = Queue()
+
+
 class TaskListView(UserListAPIView):
     queryset = TaskModel.objects.all()
     serializer_class = serializers.TaskSerializer
@@ -48,35 +63,40 @@ class ScheduleLogViewSet(ModelViewSet):
 
 
 class TaskScheduleQueueAPI:
+    queue_mapping = QueueMapping(status_params_mapping)
 
     @staticmethod
-    def query_expiring_schedules():
-        if schedule_queue_lock.locked():
-            return
-        schedule_queue_lock.acquire()
+    def query_expiring_schedules(queue, schedule_status):
         now = datetime.now()
-        try:
-            queryset = TaskSchedule.objects.filter(next_schedule_time__lte=now, status=TaskScheduleStatus.OPENING.value)
-            for schedule in queryset:
-                schedule_queue.put(serializers.QueueScheduleSerializer(schedule).data)
-                schedule.generate_next_schedule()
-            return queryset
-        finally:
-            schedule_queue_lock.release()
+        queryset = TaskSchedule.objects.filter(next_schedule_time__lte=now, status=TaskScheduleStatus.OPENING.value)
+        for schedule in queryset:
+            queue.put(serializers.QueueScheduleSerializer(schedule).data)
+            queue.generate_next_schedule()
+        return queryset
 
     @staticmethod
     @api_view(['GET'])
-    def get(request: HttpRequest):
+    def get(request: Request):
+        schedule_status = request.query_params.get('status', 'opening')
+        if schedule_status not in status_params_mapping:
+            return Response({'error': 'status must be in {}'.format(list(status_params_mapping.keys()))},
+                            status=status.HTTP_400_BAD_REQUEST)
+        queue = TaskScheduleQueueAPI.queue_mapping[schedule_status]
         try:
-            schedule = schedule_queue.get(block=False)
+            schedule = queue.get(block=False)
         except Empty:
-            TaskScheduleQueueAPI.query_expiring_schedules()
             try:
-                schedule = schedule_queue.get(block=False)
+                if schedule_queue_lock.locked():
+                    raise Empty
+                schedule_queue_lock.acquire()
+                TaskScheduleQueueAPI.query_expiring_schedules(queue, schedule_status)
+                schedule = queue.get(block=False)
             except Empty:
                 return Response({'msg': 'no schedule to run'}, status=status.HTTP_204_NO_CONTENT)
-        except Exception as e:
-            return Response({'error': str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
+            except Exception as e:
+                return Response({'error': str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
+            finally:
+                schedule_queue_lock.release()
         return Response(schedule)
 
     @staticmethod
@@ -91,7 +111,7 @@ class TaskScheduleQueueAPI:
     @staticmethod
     @api_view(['GET'])
     def size(request):
-        return Response({'size': schedule_queue.qsize()})
+        return Response({x: y.qsize() for x, y in TaskScheduleQueueAPI.queue_mapping.items()})
 
 
 class ScheduleTimeParseView(APIView):
